/**
 * Max Fighter
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://maxfighter.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include "maxfighter.h"
#include "buildinfo.h"

namespace maxfighter
{
	/** Holds the Resources instance. */
	Resources *resources;

	/** Holds the Menu instance. */
	Menu *menu;

	/** Holds the current Game instance, if any. */
	Game *game;

	/** A list of sprites which must be deleted at the end of the
	 *  current main loop. */
	std::list<mysdl::sprites::Sprite *> deleteQueue;

	/*
	* See class definition.
	*/
	MaxFighter::MaxFighter() : mysdl::FrameRateLimitedApplication()
  {
		maxTicks=specs.maxTicksPerFrame;
		desiredTicks=specs.desiredTicks-specs.desiredTicksEpsilon;

    game=0;
    resources=0;

		mysdl::debug::groups[dbgCore].name="maxfighter:core";
		mysdl::debug::groups[dbgCore].level=mysdl::debug::dlDebug;
		mysdl::debug::groups[dbgGame].name="maxfighter:game";
		mysdl::debug::groups[dbgGame].level=mysdl::debug::dlDebug;
		mysdl::debug::groups[dbgResources].name="maxfighter:resources";
		mysdl::debug::groups[dbgResources].level=mysdl::debug::dlDebug;

		mysdl::debug::info(dbgCore,"Max Fighter version %s.",getFullVersion().c_str());

		try
		{
			mysdl::init((mysdl::SubSystem)(mysdl::ssVideo|mysdl::ssJoystick));
		}
		catch(std::string ex)
    {
			mysdl::debug::critical(dbgCore,"Init() failed: %s\n",ex.c_str());
      delete this;
			return;
    }

		mysdl::randomize();

		try
		{
			resources=new Resources();
		}
		catch(std::string s)
		{
			mysdl::debug::critical(dbgCore,"Could not load resources: %s\n",s.c_str());
			delete this;
			return;
		}

		try
		{
			mysdl::audio::init();
		}
		catch(std::string ex)
		{
			mysdl::debug::critical(dbgCore,"Could not intialize mysdl:audio: %s\n",ex.c_str());
			delete this;
			return;
		}

		try
		{
			resources->setResolution(resources->settings.xRes,resources->settings.yRes,32,resources->settings.fullscreen);
		}
    catch(std::string ex)
    {
			mysdl::debug::critical(dbgCore,"InitScreen() failed: %s\n",ex.c_str());
      delete this;
			return;
    }

		mysdl::setWindowCaption("Max Fighter");
		mysdl::setWindowIcon((dirManager.getSubDir(mysdl::sdConstData,"",false)+"images_gui/icon.png").c_str());
		mysdl::showCursor(false);

		try
		{
			resources->init();
		}
		catch(std::string ex)
		{
			mysdl::debug::error(dbgCore,"Could not initialize resources: %s",ex.c_str());
			throw ex;
		}

		menu=new Menu();
		menu->activate();

		queue=resources->queue;
	}

	/*
	* See class definition.
	*/
	MaxFighter::~MaxFighter()
  {
		if(game)
    {
      delete game;
      game=0;
    }

		if(menu)
		{
			delete menu;
			menu=0;
		}

    if(resources)
    {
			resources->saveSettings();

      delete resources;
      resources=0;
    }

		mysdl::debug::debug(dbgCore,"Quitting MySDL.");
    mysdl::quit();
  }

	/* framerate delimiter....necessary?
			if(ticks>specs.maxTicksPerFrame)
			{
				ticks=specs.maxTicksPerFrame;
			}
			else if(ticks<specs.desiredTicks-specs.desiredTicksEpsilon)
			{
				SDL_Delay(specs.desiredTicks-ticks);
				ticks+=mysdl::getElapsedTicks();
			}
			*/
};
