/**
 * Max Fighter
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://maxfighter.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#ifndef MAXFIGHTER_MAXFIGHTER_H
#define MAXFIGHTER_MAXFIGHTER_H

#include <assert.h>
#include <iostream>
#include <stdio.h>
#include <string>

#include <mysdl/debug.h>
#include <mysdl/mysdl.h>
#include <mysdl/tools.h>

#include <mysdl/sprites/sprites.h>

#include "maxfighter.h"
#include "game.h"
#include "menu.h"
#include "resources.h"
#include "specs.h"

namespace maxfighter
{
	/** Holds the Resources instance. */
	extern Resources *resources;

	/** Holds the Menu instance. */
	extern Menu *menu;

	/** Holds the current Game instance, if any. */
	extern Game *game;

	/** A list of sprites which must be deleted at the end of the
	 *  current main loop. */
	extern std::list<mysdl::sprites::Sprite *> deleteQueue;

	/**
	 * The game application, a simple subclass of mysdl's framerate
	 * limited application class.
	 */
	class MaxFighter : public mysdl::FrameRateLimitedApplication
	{
		public:
			/**
			 * Constructor. Initialises hardware resources and libraries,
			 * creates the Resources instance and the Menu instance and
			 * starts the game, using the Menu instance as the initial
			 * game state.
			 */
			MaxFighter();

			/**
			 * Destructor. Releases all resources, saves the current
			 * settings and quits the game.
			 */
			~MaxFighter();
	};
};

#endif
