/**
 * Max Fighter
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://maxfighter.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include <mysdl/mysdl.h>
#include <mysdl/gui/text.h>

#include "menu.h"
#include "maxfighter.h"
#include "resources.h"

namespace maxfighter
{
	// ==================================================================
	//  Menu
	// ==================================================================

	/*
	* See class definition.
	*/
	Menu::Menu() :
		GameState(),
		state(msMenu),
		stateChangeTimer(specs.menuSwitchDuration),
		spritesHelp(0)
	{
		resources->initMenu();
		lastGameButton=resources->btnSinglePlayerA;
	}

	/*
	* See class definition.
	*/
	Menu::~Menu()
	{
		if(spritesHelp)
		{
			delete spritesHelp;
			spritesHelp=0;
		}

		if(resources->musicManager)
		{
			resources->musicManager->stop();
		}

		resources->uninitMenu();
		menu=0;
	}

	/*
	* See class definition.
	*/
	void Menu::activate()
	{
		GameState::activate();

		resources->evaluatorMenu->enabled=(resources->imagesLoaded);
		resources->gui->Evaluator::enabled=true;

		mysdl::setClearColor(mysdl::Color(0.0,0.0,0.0,0.0));
		mysdl::getElapsedTicks();

		if(resources->imageLoader)
		{
			resources->paneCredits->setVisible(true);
		}
		else
		{
			resources->paneMain->setVisible(true);
			lastGameButton->setFocused(true);
		}

		resources->gui->setActive(true);
		resources->gui->setVisible(true);

		startMusic();
	}

	/*
	* See class definition.
	*/
	void Menu::deactivate()
	{
		resources->paneMain->setVisible(false);
		resources->gui->setActive(false);
		resources->gui->setVisible(false);

		GameState::deactivate();
	}

	/*
	* See class definition.
	*/
	void Menu::startMusic()
	{
		if((resources->musicManager)&&(resources->currentVolumeMusic>0))
		{
			resources->musicManager->clearQueue();
			resources->musicManager->addAsLast(resources->musicTitleIntro);
			resources->musicManager->addAsLast(resources->musicTitleLoop);

			if(resources->musicManager->isPlaying())
			{
				resources->musicManager->skipToNext();
			}
			else
			{
				resources->musicManager->play();
			}
		}
	}

	/*
	* See class definition.
	*/
	void Menu::closeWindowSoundSettings(bool confirm)
	{
		GameState::closeWindowSoundSettings(confirm);
		resources->paneSettings->setVisible(true);
		resources->btnSoundSettings->setFocused(true);
	}

	/*
	* See class definition.
	*/
	void Menu::closeWindowControls(bool confirm)
	{
		GameState::closeWindowControls(confirm);
		resources->paneSettings->setVisible(true);

		if(originalEvaluator==resources->evaluatorPlayerSingle[0])
		{
			resources->btnControlsSinglePlayer->setFocused(true);
		}
		else if(originalEvaluator==resources->evaluatorPlayerMulti[0])
		{
			resources->btnControlsMultiplayer[0]->setFocused(true);
		}
		else if(originalEvaluator==resources->evaluatorPlayerMulti[1])
		{
			resources->btnControlsMultiplayer[1]->setFocused(true);
		}
	}

	/*
	* See class definition.
	*/
	void Menu::handleEvent(const mysdl::Trigger *e)
	{
		switch(e->id)
		{
			case cgOk:
			{
				if(resources->paneQuit->getVisible())
				{
					terminate=true;
					break;
				}
				// otherwise fall through
			}
			
			case cgExit:
			{
				if(state>msMenu)
				{
					state=msMenu;
					resources->gui->setActive(true);
					resources->gui->setVisible(true);
					stateChangeTimer=specs.menuSwitchDuration;
					break;
				}

				if(closeWindows(e->id==cgOk))
				{
					break;
				}

				if(resources->paneGetKey->getVisible())
				{
				}
				else if(resources->paneSettings->getVisible())
				{
					if(controlsChanged)
					{
						resources->saveSettings();
						controlsChanged=false;
					}

					resources->paneSettings->setVisible(false);
					resources->paneMain->setVisible(true);
					resources->btnSettings->setFocused(true);
				}
				else if((resources->paneCredits->getVisible())&&(resources->imagesLoaded))
				{
					resources->paneCredits->setVisible(false);
					resources->paneMain->setVisible(true);
					resources->btnCredits->setFocused(true);
				}
				else if(resources->paneQuit->getVisible())
				{
					resources->paneQuit->setVisible(false);
					resources->paneMain->setVisible(true);
					resources->btnQuit->setFocused(true);
				}
				else if(resources->imagesLoaded)
				{
					resources->paneQuit->setVisible(true);
					resources->paneMain->setVisible(false);
					resources->btnQuitNo->setFocused(true);
				}
				else
				{
					terminate=true;
				}

				break;
			}

			case cmSettings:
			{
				resources->paneSettings->setVisible(true);
				resources->paneMain->setVisible(false);
				resources->btnSettingsDone->setFocused(true);

				break;
			}

			case cmCredits:
			{
				resources->paneCredits->setVisible(true);
				resources->paneMain->setVisible(false);
				resources->paneCredits->setFocused(true);

				break;
			}

			case cmSinglePlayerA:
			{
				lastGameButton=resources->btnSinglePlayerA;

				game=new Game(gtSingleA);
				game->activate();

				break;
			}

			case cmSinglePlayerB:
			{
				lastGameButton=resources->btnSinglePlayerB;

				game=new Game(gtSingleB);
				game->activate();

				break;
			}

			case cmMultiplayer:
			{
				lastGameButton=resources->btnMultiplayer;

				game=new Game(gtMulti);
				game->activate();

				break;
			}

			default:
			{
				GameState::handleEvent(e);
			}
		}
	}

	/*
	* See class definition.
	*/
	void Menu::doDraw(int stage)
	{
		switch(stage)
		{
			case 1:
			{
				if(resources->gui->getVisible())
				{
					if(!(resources->paneControls->getVisible()))
					{
						resources->contextMenu.apply();
						resources->logoColor.apply();
						resources->imgLogo->draw(resources->logoPosition);
					}
				}

				break;
			}

			case 2:
			{
				if(state==msScoresSingle)
				{
					mysdl::Color(1.0,1.0,1.0,1.0).apply();

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(400,530));
					resources->fontStandard->printf(mysdl::jhCenter,"----- H I G H S C O R E S -----");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(200,430));
					resources->fontStandard->printf(mysdl::jhCenter,"Single Player Mode A");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,430));
					resources->fontStandard->printf(mysdl::jhCenter,"Single Player Mode B");

					displayHighscores(200,230,&(resources->highscores.singleA),-1,-1);
					displayHighscores(600,230,&(resources->highscores.singleB),-1,-1);
				}
				else if(state==msScoresMulti)
				{
					mysdl::Color(1.0,1.0,1.0,1.0).apply();

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(400,530));
					resources->fontStandard->printf(mysdl::jhCenter,"----- H I G H S C O R E S -----");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(400,430));
					resources->fontStandard->printf(mysdl::jhCenter,"Multiplayer");

					displayHighscores(400,230,&(resources->highscores.multi),-1,-1);
				}
				else if(state==msHelp)
				{
					mysdl::Color(1.0,1.0,1.0,1.0).apply();

					mysdl::resetCursor();
					resources->imgHudFull->draw(specs.hudPosPlayer1);

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(100,530));
					resources->fontStandard->printf(mysdl::jhLeft,"Upgrade");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(300,530));
					resources->fontStandard->printf(mysdl::jhLeft,"Upgrade");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(500,530));
					resources->fontStandard->printf(mysdl::jhLeft,"Upgrade");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(150,430));
					resources->fontStandard->printf(mysdl::jhLeft,"Energy");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(350,430));
					resources->fontStandard->printf(mysdl::jhLeft,"Weapon");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(200,330));
					resources->fontStandard->printf(mysdl::jhLeft,"Container");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(400,330));
					resources->fontStandard->printf(mysdl::jhLeft,"Container");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,330));
					resources->fontStandard->printf(mysdl::jhLeft,"Container");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(250,180));
					resources->fontStandard->printf(mysdl::jhLeft,"Small Asteroid");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,180));
					resources->fontStandard->printf(mysdl::jhLeft,"Large Asteroid");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(250,80));
					resources->fontStandard->printf(mysdl::jhLeft,"Pirate");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,80));
					resources->fontStandard->printf(mysdl::jhLeft,"Crystal");

					mysdl::Color(1.0,1.0,0.0,1.0).apply();

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(100,500));
					resources->fontStandard->printf(mysdl::jhLeft,"Speed");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(300,500));
					resources->fontStandard->printf(mysdl::jhLeft,"Fire Rate");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(500,500));
					resources->fontStandard->printf(mysdl::jhLeft,"Laser Type");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(150,400));
					resources->fontStandard->printf(mysdl::jhLeft,"Shield");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(350,400));
					resources->fontStandard->printf(mysdl::jhLeft,"Time Warp");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(200,300));
					resources->fontStandard->printf(mysdl::jhLeft,"Upgrade");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(400,300));
					resources->fontStandard->printf(mysdl::jhLeft,"Energy");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,300));
					resources->fontStandard->printf(mysdl::jhLeft,"Weapon");

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(250,150));
					resources->fontStandard->printf(mysdl::jhLeft,"%d pts",specs.scoreAsteroidSmall);

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,150));
					resources->fontStandard->printf(mysdl::jhLeft,"%d pts",specs.scoreAsteroidLarge);

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(250,50));
					resources->fontStandard->printf(mysdl::jhLeft,"%d pts",specs.scoreFoe);

					mysdl::resetCursor();
					mysdl::moveCursor(mysdl::Vector2D(600,50));
					resources->fontStandard->printf(mysdl::jhLeft,"%d pts",specs.scoreCrystal);

					mysdl::resetCursor();
					spritesHelp->draw();
				}
			}
		}

		GameState::doDraw(stage);
	}

	/*
	* See class definition.
	*/
	void Menu::initSpritesHelp()
	{
		mysdl::sprites::ImageSprite *sprite;
		spritesHelp=new mysdl::sprites::SpriteEngine();

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(50,530),true);
		sprite->setAnimation(resources->animExtraSpeedup);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(250,530),true);
		sprite->setAnimation(resources->animExtraFireRate);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(450,530),true);
		sprite->setAnimation(resources->animExtraLaserType);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(100,430),true);
		sprite->setAnimation(resources->animExtraShield);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(300,430),true);
		sprite->setAnimation(resources->animExtraTimeWarp);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(150,330),true);
		sprite->setAnimation(resources->animContainer[0]);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(350,330),true);
		sprite->setAnimation(resources->animContainer[1]);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(550,330),true);
		sprite->setAnimation(resources->animContainer[2]);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(190,190),true);
		sprite->setAnimation(resources->animAsteroidSmall);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(520,180),true);
		sprite->setAnimation(resources->animAsteroidLarge);
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(190,80),true);
		sprite->setAnimation(resources->animFoeLeft)->target=0;
		spritesHelp->addBack(sprite);

		sprite=new mysdl::sprites::ImageSprite(mysdl::Vector2D(540,80),true);
		sprite->setAnimation(resources->animExtraScore);
		spritesHelp->addBack(sprite);
	}

	/*
	* See class definition.
	*/
	void Menu::doPerform(int stage,Uint32 ticks)
	{
		switch(stage)
		{
			case 0:
			{
				if(!resources->imageLoader)
				{
					if(state==msHelp)
					{
						spritesHelp->perform(ticks);
					}

					if(stateChangeTimer>ticks)
					{
						stateChangeTimer-=ticks;
					}
					else
					{
						if(state==msMenu)
						{
							state=msScoresSingle;
							resources->gui->setActive(false);
							resources->gui->setVisible(false);
							stateChangeTimer=specs.menuHighscoreDuration;
						}
						else if(state==msScoresSingle)
						{
							state=msScoresMulti;
							stateChangeTimer=specs.menuHighscoreDuration;
						}
						else if(state==msScoresMulti)
						{
							state=msHelp;
							stateChangeTimer=specs.menuHelpStepDuration;
						}
						else if(state==msHelp)
						{
							state=msMenu;
							resources->gui->setActive(true);
							resources->gui->setVisible(true);
							stateChangeTimer=specs.menuSwitchDuration;
						}
					}
				}
			}

			case 1:
			{
				resources->gui->Component::perform(resources->queue,ticks);

				if(resources->starfield->speedFactor<specs.starfieldMenuSpeed)
				{
					resources->starfield->speedFactor+=specs.starfieldMenuSpeedStep*ticks;

					if(resources->starfield->speedFactor>specs.starfieldMenuSpeed)
					{
						resources->starfield->speedFactor=specs.starfieldMenuSpeed;
					}
				}
				else if(resources->starfield->speedFactor>specs.starfieldMenuSpeed)
				{
					resources->starfield->speedFactor-=specs.starfieldMenuSpeedStep*ticks;

					if(resources->starfield->speedFactor<specs.starfieldMenuSpeed)
					{
						resources->starfield->speedFactor=specs.starfieldMenuSpeed;
					}
				}

				if(resources->imageLoader)
				{
					resources->logoColor.a=resources->imageLoader->getProgressRelative();
				}
			}
		}

		GameState::doPerform(stage,ticks);
	}
};
