/**
 * Max Fighter
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://maxfighter.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#ifndef MAXFIGHTER_MENU_H
#define MAXFIGHTER_MENU_H

#include "resources.h"

namespace maxfighter
{
	/** Possible states for the Menu state machine. */
	enum MenuState {msMenu,msScoresSingle,msScoresMulti,msHelp};

	/**
	 * A game state representing the main menu and its sub menus. The Menu state
	 * is itself a state machine which may switch to alternate states e.g. for
	 * showing highscores or game instructions.
	 */
	class Menu : public GameState
	{
		public:
			/** Current state. */
			MenuState state;

			/** Number of milliseconds until next state change. */
			Uint32 stateChangeTimer;

			/** Sprites for the game instructions state. */
			mysdl::sprites::SpriteEngine *spritesHelp;

			/** The button that was used to start the latest game. Used to
			 *  reactivate that button after going back to the title menu. */
			mysdl::gui::Button *lastGameButton;

			/**
			 * Constructor.
			 */
			Menu();

			/**
			 * Destructor.
			 */
			~Menu();

			/**
			 * Activates the menu.
			 */
			void activate();

			/**
			 * Deactivates the menu when switching to a different game
			 * state.
			 */
			void deactivate();

			/**
			 * Handles menu events.
			 */
			void handleEvent(const mysdl::Trigger *e);

			/**
			 * Performs menu state updates. Handles the substate switch
			 * counter as well as the slowing down of the starfield speed
			 * after a game.
			 */
			void doPerform(int stage,Uint32 ticks);

			/**
			 * Starts the main title music if music playback is enabled in
			 * the settings.
			 */
			void startMusic();

			/**
			 * Draws the current menu state.
			 * In stage 1, the logo is drawn if the menu is currently active.
			 * In stage 2, any current non-menu state is drawn.
			 */
			void doDraw(int stage);

			/**
			 * Returns to the settings menu after leaving the sound settings
			 * menu.
			 */
			void closeWindowSoundSettings(bool confirm);

			/**
			 * Returns to the settings menu after leaving the control
			 * configuration menu.
			 */
			void closeWindowControls(bool confirm);

			/**
			 * Initialises the sprite engine for the game instructions state.
			 */
			void initSpritesHelp();
	};
};

#endif
