/**
 * Max Fighter
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://maxfighter.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include <mysdl/gui/standard.h>
#include <mysdl/gui/text.h>

#include "buildinfo.h"
#include "maxfighter.h"
#include "resources.h"

namespace maxfighter
{
	/** Filename (in current path) for saving a screenshot. */
	const char *FILE_SCREENSHOT="screenshot.png";

	/** Directory containing the resource files. TODO: how does this work with the directory manager? */
	const std::string resourceDir="./resources/";

	/** The key under which the directory manager stores/reads resource data. */
	const std::string GAME_KEY="maxfighter";

	/** Tells the application where to find its various files on the current system. */
	mysdl::SysDirManager dirManager(GAME_KEY,mysdl::sdGame);

	/** Name of the highscores file (global, one for all users). */
	static const std::string FILE_HIGHSCORES="highscores.xml";

	/** Name of the settings file (one for each users). */
	static const std::string FILE_SETTINGS="settings.xml";

	/** Strings for the resources file. */
	static const std::string XML_SETTINGS_NAME             ="name";
	static const std::string XML_SETTINGS_CONTROLS         ="controls";
	static const std::string XML_SETTINGS_CONTROLS_SINGLE  ="single";
	static const std::string XML_SETTINGS_CONTROLS_MULTI_1 ="multi 1";
	static const std::string XML_SETTINGS_CONTROLS_MULTI_2 ="multi 2";
	static const std::string XML_SETTINGS_SCREEN           ="screen";
	static const std::string XML_SETTINGS_FULLSCREEN       ="fullscreen";
	static const std::string XML_SETTINGS_XRES             ="xRes";
	static const std::string XML_SETTINGS_YRES             ="yRes";
	static const std::string XML_SETTINGS_SOUND            ="sound";
	static const std::string XML_SETTINGS_SFX              ="sfx";
	static const std::string XML_SETTINGS_MUSIC            ="music";
	static const std::string XML_SETTINGS_VOLUME_SFX       ="volumeSfx";
	static const std::string XML_SETTINGS_VOLUME_MUSIC     ="volumeMusic";

	/** Strings for the highscores file. */
	static const std::string XML_HIGHSCORES         ="highscores";
	static const std::string XML_HIGHSCORES_SINGLE_A="singleA";
	static const std::string XML_HIGHSCORES_SINGLE_B="singleB";
	static const std::string XML_HIGHSCORES_MULTI   ="multi";

	/** Default name for predefined highscore entries. */
	static const std::string HIGHSCORES_DEFAULT_NAME="-----------------";

	/** Connected to the GUI to play sounds for menu events. */
	GUICallback callback;

	/*
	* See class definition.
	*/
	Settings::Settings() :
		xRes(RES_X),
		yRes(RES_Y),
		fullscreen(DEF_FULLSCREEN),
		volumeSfx(70),
		volumeMusic(100)
	{
	}

	// ==================================================================
	//  Resources
	// ==================================================================

	/*
	* See class definition.
	*/
	Resources::Resources() :
		mysdl::ResourceManager(),
		imagesLoaded(false),
		vpPlayfield(),contextGame(),contextHud(),contextMenu(),
		fonts(0),fontStandard(0),
		imagesGame(0),animationsGame(0),
		imagesHud(0),imagesGui(0),
		gui(0),
		evaluatorCore(0),evaluatorMenu(0),queue(0),
		colorWhite(1,1,1,1),
		starfieldProperties(),starfield(0),
		samples(0),music(0),
		/*sampleBeep(0),*/sampleExplosion(0),sampleLaser(0),samplePing(0),sampleExtra(0),sampleTimeWarp(0),sampleMenuMove(0),sampleMenuClick(0),
		musicTitleIntro(0),musicTitleLoop(0),
		musicGameAIntro(0),musicGameALoop(0),musicGameBIntro(0),musicGameBLoop(0),musicDeath(0),
		musicHighscoresIntro(0),musicHighscoresLoop(0),musicGameOverIntro(0),musicGameOverLoop(0),
		sourcesGeneral(0),sourcesExplosions(0),
		musicManager(0),
		highscores(),
		imgLogo(0),
		animPlayer(),
		imgLaserRay(0),imgLaserRound(0),imgLaserFoe(0),
		animEngine(0),animEngineStutter(0),animShield(0),animAsteroidSmall(0),animAsteroidLarge(0),animCrystal(0),
		animContainer(),
		animExtraFireRate(0),animExtraLaserType(0),animExtraSpeedup(0),animExtraShield(0),animExtraScore(0),
		animExtraTimeWarp(0),
		animFoeLeft(0),animFoeRight(0),
		animExplosionFast(0),animExplosionSlow(0),
		imgHudFull(0),imgHudShield(0),imgHudSpeed(),imgHudFire(),imgHudTimewarp(),
		imageLoader(0),
		paneMain(0),paneSettings(0),paneControls(0),paneCredits(0),paneSoundSettings(0),paneEnterName(0),paneInGame(0),paneQuit(0),
		btnQuitNo(0),btnQuit(0),btnSinglePlayerA(0),btnSinglePlayerB(0),btnMultiplayer(0),btnSettings(0),btnCredits(0),
		btnControlsSinglePlayer(0),btnControlsMultiplayer(),btnSettingsDone(0),
		btnSoundSettings(0),btnSoundSettingsOk(0),
		btnControlsPlayer1(0),btnControlsPlayer2(0),btnControlsLeft(0),btnControlsRight(0),btnControlsFire(0),
		labelControlsHeader(0),labelControlsLeft(0),labelControlsRight(0),labelControlsFire(0),
		labelHeaderVersion(0),
		btnContinue(0),btnInGameSoundSettings(0),btnInGameControlsSinglePlayer(0),btnInGameControlsMultiplayer(),
		sVolumeSounds(0),sVolumeMusic(0),
		lVolumeSounds(0),lVolumeMusic(0),
		lsTextfieldRed(0),lsTextfieldBlue(0),
		logoPosition(),logoColor(1.0,1.0,1.0,0.0)
	{
		#ifdef RESOURCEDIR
			dirManager.constDataDir=RESOURCEDIR;
		#endif

		load(dirManager.getSubDir(mysdl::sdConstData,"",false)+"resources.xml");

		joystick[0]=joystick[1]=0;

		if(!(fonts=dynamic_cast<mysdl::fonts::FontBank *>((*this)["fonts"])))
		{
			throw std::string("fonts not found");
		}

		if(!(imagesGame=dynamic_cast<mysdl::gfx::ImageBank *>((*this)["imagesGame"])))
		{
			throw std::string("imagesGame not found");
		}

		if(!(animationsGame=dynamic_cast<mysdl::gfx::AnimationBank *>((*this)["animationsGame"])))
		{
			throw std::string("animationsGame not found");
		}

		if(!(imagesHud=dynamic_cast<mysdl::gfx::ImageBank *>((*this)["imagesHud"])))
		{
			throw std::string("imagesHud not found");
		}

		if(!(imagesGui=dynamic_cast<mysdl::gfx::ImageBank *>((*this)["imagesGui"])))
		{
			throw std::string("imagesGui not found");
		}

		if(!(gui=dynamic_cast<mysdl::gui::GUI *>((*this)["gui"])))
		{
			throw std::string("gui not found");
		}

		if(!(samples=dynamic_cast<mysdl::audio::SampleBank *>((*this)["sounds"])))
		{
			throw std::string("samples not found");
		}

		if(!(music=dynamic_cast<mysdl::audio::SampleBank *>((*this)["music"])))
		{
			throw std::string("music not found");
		}

		joystick[0]=mysdl::initJoystick(0);
		joystick[1]=mysdl::initJoystick(1);

		if(!(evaluatorCore=dynamic_cast<mysdl::InputEvaluator *>((*this)["evaluatorCore"])))
		{
			throw std::string("evaluatorCore not found");
		}

		if(!(evaluatorMenu=dynamic_cast<mysdl::InputEvaluator *>((*this)["evaluatorMenu"])))
		{
			throw std::string("evaluatorMenu not found");
		}

		if(!(evaluatorGameCore=dynamic_cast<mysdl::InputEvaluator *>((*this)["evaluatorGameCore"])))
		{
			throw std::string("evaluatorGameCore not found");
		}

		rename("evaluatorGamePlayer","evaluatorGamePlayerSingle[0]");

		if(!(evaluatorPlayerSingle[0]=dynamic_cast<mysdl::InputEvaluator *>((*this)["evaluatorGamePlayerSingle[0]"])))
		{
			throw std::string("evaluatorGamePlayerSingle[0] not found");
		}

		evaluatorPlayerSingle[0]->setDataOnAllTriggers(0); // set player index
		
		evaluatorPlayerMulti[0]=evaluatorPlayerSingle[0]->clone();
		evaluatorPlayerMulti[0]->setDataOnAllTriggers(0); // set player index
		this->add("evaluatorGamePlayerMulti[0]",evaluatorPlayerMulti[0]);

		evaluatorPlayerMulti[1]=evaluatorPlayerSingle[0]->clone();
		evaluatorPlayerMulti[1]->setDataOnAllTriggers(1); // set player index
		this->add("evaluatorGamePlayerMulti[1]",evaluatorPlayerMulti[1]);

		queue=&mysdl::queue;
		queue->deactivateTrigger=(*evaluatorCore)["pause"];

		evaluatorGameCore->enabled=false;
		evaluatorPlayerSingle[0]->enabled=false;
		evaluatorPlayerMulti[0]->enabled=false;
		evaluatorPlayerMulti[1]->enabled=false;

		// why??
		queue->evaluators.push_front(evaluatorCore);
		queue->evaluators.push_front(evaluatorMenu);

		queue->evaluators.push_front(evaluatorGameCore);
		queue->evaluators.push_front(evaluatorPlayerSingle[0]);
		queue->evaluators.push_front(evaluatorPlayerMulti[0]);
		queue->evaluators.push_front(evaluatorPlayerMulti[1]);

		queue->evaluators.push_front(gui);

		loadSettings();

		contextGame.blending=true;
		contextHud.blending=true;
		contextHud.blendMode=mysdl::bmAdd;
		contextMenu.blending=true;

		mysdl::particles::StarClass sc1(specs.starCount/5,0.2,mysdl::Color(0.2,0.2,0.2,1.0));
		mysdl::particles::StarClass sc2(specs.starCount/5,0.4,mysdl::Color(0.4,0.4,0.4,1.0));
		mysdl::particles::StarClass sc3(specs.starCount/5,0.6,mysdl::Color(0.6,0.6,0.6,1.0));
		mysdl::particles::StarClass sc4(specs.starCount/5,0.8,mysdl::Color(0.8,0.8,0.8,1.0));
		mysdl::particles::StarClass sc5(specs.starCount/5,1.0,mysdl::Color(1.0,1.0,1.0,1.0));

		starfieldProperties.classes.push_back(sc1);
		starfieldProperties.classes.push_back(sc2);
		starfieldProperties.classes.push_back(sc3);
		starfieldProperties.classes.push_back(sc4);
		starfieldProperties.classes.push_back(sc5);

		imgHudFull=(*imagesHud)["full"];
		imgHudShield=(*imagesHud)["shield"];

		for(int i=0; i<5; i++)
		{
			imgHudSpeed[i]=(*imagesHud)["speed_"+mysdl::toStr(i+1)];

			for(int j=0; j<6; j++)
			{
				imgHudFire[i][j]=(*imagesHud)["fire_"+mysdl::toStr(i+1)+"_"+mysdl::toStr(j)];
			}
		}

		for(int i=0; i<specs.timeWarpLevelMax; i++)
		{
			imgHudTimewarp[i]=(*imagesHud)["timewarp_"+mysdl::toStr(i+1)];
		}

		imgLogo=(*imagesGui)["logo"];

		animPlayer[0]=(*animationsGame)["player1"];
		animPlayer[1]=(*animationsGame)["player2"];

		imgLaserRay=(*imagesGame)["laserRayGreen"];
		imgLaserRound=(*imagesGame)["laserRoundGreen"];
		imgLaserFoe=(*imagesGame)["laserRoundRed"];

		animEngine=(*animationsGame)["engine"];
		animEngineStutter=(*animationsGame)["engineStutter"];
		animShield=(*animationsGame)["shield"];
		animAsteroidSmall=(*animationsGame)["asteroidSmall"];
		animAsteroidLarge=(*animationsGame)["asteroidLarge"];
		animCrystal=(*animationsGame)["crystal"];

		animContainer[0]=(*animationsGame)["container_0"];
		animContainer[1]=(*animationsGame)["container_1"];
		animContainer[2]=(*animationsGame)["container_2"];

		animExtraFireRate=(*animationsGame)["extraFireRate"];
		animExtraLaserType=(*animationsGame)["extraLaserType"];
		animExtraSpeedup=(*animationsGame)["extraSpeedup"];
		animExtraShield=(*animationsGame)["extraShield"];
		animExtraScore=(*animationsGame)["extraScore"];

		animExtraTimeWarp=(*animationsGame)["extraTimeWarp"];

		animFoeLeft=(*animationsGame)["foeLeft"];
		animFoeRight=(*animationsGame)["foeRight"];

		animExplosionFast=(*animationsGame)["explosionFast"];
		animExplosionSlow=(*animationsGame)["explosionSlow"];
	}

	/*
	* See class definition.
	*/
	Resources::~Resources()
	{
		mysdl::secureDelete(sourcesExplosions);
		mysdl::secureDelete(musicManager);
		mysdl::secureDelete(starfield);

		for(int i=0; i<2; i++)
		{
			if(joystick[i])
			{
				mysdl::cleanUpJoystick(joystick[i]);
				joystick[i]=0;
			}
		}
	}

	/*
	* See class definition.
	*/
	void Resources::setResolution(int width,int height,int depth,bool fullscreen)
	{
		mysdl::setResolution(width,height,depth,fullscreen);

		contextGame.virtualDimensions.x=RES_X;
		contextGame.virtualDimensions.y=RES_Y;

		contextHud.virtualDimensions.x=RES_X;
		contextHud.virtualDimensions.y=RES_Y;

		contextMenu.virtualDimensions.x=width;
		contextMenu.virtualDimensions.y=height;

		vpPlayfield.dimensions=contextGame.virtualDimensions;
		starfieldProperties.dimensions=contextGame.virtualDimensions;

		#ifdef LOSSY_SURFACES
			imagesHud->reload();
			imagesHud->refresh(0);
		
			imagesGui->reload();
			imagesGui->refresh(0);

			if(imagesLoaded)
			{
				imagesGame->reload();
				imagesGame->refresh(0); // TODO: why reload AND refresh??
			}
			
			fonts->refresh();
		#endif
	}

	/*
	* See class definition.
	*/
	void Resources::saveHighscores()
	{
		try
		{
			highscores.save(dirManager.getSubDir(mysdl::sdVarData,"",true)+FILE_HIGHSCORES);
		}
		catch(std::string ex)
		{
			mysdl::debug::info(dbgResources,"Could not save highscores: %s",ex.c_str());
		}
	}

	/*
	* See class definition.
	*/
	void Resources::onStart(mysdl::BackgroundTask *task)
	{
	}

	/*
	* See class definition.
	*/
	void Resources::onFinish(mysdl::BackgroundTask *task)
	{
		paneCredits->setVisible(false);
		paneMain->setVisible(true);
		btnSinglePlayerA->setFocused(true);

		imageLoader=0;
		imagesLoaded=true;
		evaluatorMenu->enabled=true;

		menu->initSpritesHelp();
		((MaxFighter*)(mysdl::application))->desiredTicks=specs.desiredTicks-specs.desiredTicksEpsilon;
		
		//joyEscapeTranslator=new mysdl::JoyButtonTranslator((*evaluatorCore)["exit"],-1,-1,mysdl::ctrlDown);
	}

	/*
	* See class definition.
	*/
	void Resources::initHighscores()
	{
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,100000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,50000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,25000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,10000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,8000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,6000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,4000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,2000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,1000));
		highscores.singleA.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,500));

		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,20000));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,17500));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,15000));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,12500));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,10000));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,7500));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,5000));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,2500));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,1000));
		highscores.singleB.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,500));

		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,100000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,50000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,25000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,10000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,8000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,6000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,4000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,2000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,1000));
		highscores.multi.add(HighscoreEntry(HIGHSCORES_DEFAULT_NAME,500));
	}

	/*
	* See class definition.
	*/
	void Resources::init()
	{
		initHighscores();

		try
		{
			highscores.load(dirManager.getSubDir(mysdl::sdVarData,"",false)+FILE_HIGHSCORES);
		}
		catch(std::string ex)
		{
			mysdl::debug::info(dbgResources,"Could not load highscores: %s",ex.c_str());
		}

		imagesHud->refresh(0);
		imagesGui->refresh(0);

		mysdl::basic_Vector2D<GLfloat> dim=resources->contextMenu.virtualDimensions;
		logoPosition.x=(dim.x-resources->imgLogo->getWidth())/2;
		logoPosition.y=(dim.y-resources->imgLogo->getHeight())-10;

		fonts->refresh();
		fontStandard=(*fonts)[mysdl::fonts::FontDescriptor("Standard",33)];
		
		if(!fontStandard)
		{
			mysdl::debug::error(dbgResources,"Standard font not found.");
		}

		music->refresh();

		musicTitleIntro=(mysdl::audio::StreamedSample *)(*music)["music_title_intro"];
		musicTitleLoop=(mysdl::audio::StreamedSample *)(*music)["music_title_loop"];
		musicGameAIntro=(mysdl::audio::StreamedSample *)(*music)["music_game_a_intro"];
		musicGameALoop=(mysdl::audio::StreamedSample *)(*music)["music_game_a_loop"];
		musicGameBIntro=(mysdl::audio::StreamedSample *)(*music)["music_game_b_intro"];
		musicGameBLoop=(mysdl::audio::StreamedSample *)(*music)["music_game_b_loop"];
		musicDeath=(mysdl::audio::StreamedSample *)(*music)["music_death"];
		musicHighscoresIntro=(mysdl::audio::StreamedSample *)(*music)["music_highscores_intro"];
		musicHighscoresLoop=(mysdl::audio::StreamedSample *)(*music)["music_highscores_loop"];
		musicGameOverIntro=(mysdl::audio::StreamedSample *)(*music)["music_gameover_intro"];
		musicGameOverLoop=(mysdl::audio::StreamedSample *)(*music)["music_gameover_loop"];

		samples->refresh();

		sampleExplosion=(*samples)["explosion"];
		sampleLaser=(*samples)["laser"];
		samplePing=(*samples)["ping"];
		sampleExtra=(*samples)["extra"];
		sampleTimeWarp=(*samples)["timewarp"];
		sampleMenuMove=(*samples)["menu_move"];
		sampleMenuClick=(*samples)["menu_click"];

		if(settings.volumeMusic>0)
		{
			musicManager=new mysdl::audio::MusicManager();
		}
		
		sourcesGeneral=new mysdl::audio::SourcePool(2);
		sourcesExplosions=new mysdl::audio::SourcePool(10);

		setVolumeSfx(settings.volumeSfx);
		setVolumeMusic(settings.volumeMusic);

		starfieldProperties.dimensions=mysdl::Vector2D(RES_X,RES_Y);
		starfield=starfieldProperties.create();
		starfield->speedFactor=specs.starfieldMenuSpeed;

		paneMain=(mysdl::gui::Pane *)((*gui)["pMainMenu"]);
		paneSettings=(mysdl::gui::Pane *)((*gui)["pSettings"]);
		paneCredits=(mysdl::gui::Pane *)((*gui)["pCredits"]);
		paneControls=(mysdl::gui::Pane *)((*gui)["pControls"]);
		paneGetKey=(mysdl::gui::Pane *)((*gui)["pGetKey"]);
		paneSoundSettings=(mysdl::gui::Pane *)((*gui)["pSoundSettings"]);
		paneEnterName=(mysdl::gui::Pane *)((*gui)["pEnterName"]);
		paneInGame=(mysdl::gui::Pane *)((*gui)["pInGame"]);
		paneQuit=(mysdl::gui::Pane *)((*gui)["pQuit"]);
		btnQuitNo=(mysdl::gui::Button *)((*gui)["btnQuitNo"]);
		btnQuit=(mysdl::gui::Button *)((*gui)["btnQuit"]);
		btnSinglePlayerA=(mysdl::gui::Button *)((*gui)["btnSinglePlayerA"]);
		btnSinglePlayerB=(mysdl::gui::Button *)((*gui)["btnSinglePlayerB"]);
		btnMultiplayer=(mysdl::gui::Button *)((*gui)["btnMultiplayer"]);
		btnSettings=(mysdl::gui::Button *)((*gui)["btnSettings"]);
		btnCredits=(mysdl::gui::Button *)((*gui)["btnCredits"]);
		btnSettingsDone=(mysdl::gui::Button *)((*gui)["btnSettingsDone"]);
		btnSoundSettings=(mysdl::gui::Button *)((*gui)["btnSoundSettings"]);
		btnSoundSettingsOk=(mysdl::gui::Button *)((*gui)["btnSoundSettingsOk"]);
		btnControlsSinglePlayer=(mysdl::gui::Button *)((*gui)["btnControlsSinglePlayer"]);
		btnControlsMultiplayer[0]=(mysdl::gui::Button *)((*gui)["btnControlsMultiplayer1"]);
		btnControlsMultiplayer[1]=(mysdl::gui::Button *)((*gui)["btnControlsMultiplayer2"]);
		cbFullscreen=(mysdl::gui::Checkbox *)((*gui)["cbFullscreen"]);
		labelHeaderVersion=(mysdl::gui::Label *)((*gui)["lHeaderVersion"]);
		btnControlsPlayer1=(mysdl::gui::Button *)((*gui)["btnControlsPlayer1"]);
		btnControlsPlayer2=(mysdl::gui::Button *)((*gui)["btnControlsPlayer2"]);
		btnControlsLeft=(mysdl::gui::Button *)((*gui)["btnControlsLeft"]);
		btnControlsRight=(mysdl::gui::Button *)((*gui)["btnControlsRight"]);
		btnControlsFire=(mysdl::gui::Button *)((*gui)["btnControlsFire"]);
		labelControlsHeader=(mysdl::gui::Label *)((*gui)["labelControlsHeader"]);
		labelControlsLeft=(mysdl::gui::Label *)((*gui)["labelControlsLeft"]);
		labelControlsLeftJoy=(mysdl::gui::Label *)((*gui)["labelControlsLeftJoy"]);
		labelControlsRight=(mysdl::gui::Label *)((*gui)["labelControlsRight"]);
		labelControlsRightJoy=(mysdl::gui::Label *)((*gui)["labelControlsRightJoy"]);
		labelControlsUp=(mysdl::gui::Label *)((*gui)["labelControlsUp"]);
		labelControlsUpJoy=(mysdl::gui::Label *)((*gui)["labelControlsUpJoy"]);
		labelControlsDown=(mysdl::gui::Label *)((*gui)["labelControlsDown"]);
		labelControlsDownJoy=(mysdl::gui::Label *)((*gui)["labelControlsDownJoy"]);
		labelControlsFire=(mysdl::gui::Label *)((*gui)["labelControlsFire"]);
		labelControlsFireJoy=(mysdl::gui::Label *)((*gui)["labelControlsFireJoy"]);
		labelControlsShield=(mysdl::gui::Label *)((*gui)["labelControlsShield"]);
		labelControlsShieldJoy=(mysdl::gui::Label *)((*gui)["labelControlsShieldJoy"]);
		labelControlsTimeWarp=(mysdl::gui::Label *)((*gui)["labelControlsTimeWarp"]);
		labelControlsTimeWarpJoy=(mysdl::gui::Label *)((*gui)["labelControlsTimeWarpJoy"]);
		labelControlsPauseJoy=(mysdl::gui::Label *)((*gui)["labelControlsPauseJoy"]);
		labelGetKeyTriggerName=(mysdl::gui::Label *)((*gui)["lGetKeyTriggerName"]);
		labelGetKeyHeader=(mysdl::gui::Label *)((*gui)["lGetKeyHeader"]);
		labelGetKeyHeaderAlternate=(mysdl::gui::Label *)((*gui)["lGetKeyHeaderAlternate"]);
		icGetKey=(mysdl::gui::InputCaptor *)((*gui)["icGetKey"]);
		btnContinue=(mysdl::gui::Button *)((*gui)["btnContinue"]);
		cbInGameFullscreen=(mysdl::gui::Checkbox *)((*gui)["cbInGameFullscreen"]);
		btnInGameSoundSettings=(mysdl::gui::Button *)((*gui)["btnInGameSoundSettings"]);
		btnInGameControlsSinglePlayer=(mysdl::gui::Button *)((*gui)["btnInGameControlsSinglePlayer"]);
		btnInGameControlsMultiplayer[0]=(mysdl::gui::Button *)((*gui)["btnInGameControlsMultiplayer1"]);
		btnInGameControlsMultiplayer[1]=(mysdl::gui::Button *)((*gui)["btnInGameControlsMultiplayer2"]);
		sVolumeSounds=(mysdl::gui::Slider<int> *)((*gui)["sVolumeSounds"]);
		sVolumeMusic=(mysdl::gui::Slider<int> *)((*gui)["sVolumeMusic"]);
		lVolumeSounds=(mysdl::gui::Label *)((*gui)["lVolumeSoundsValue"]);
		lVolumeMusic=(mysdl::gui::Label *)((*gui)["lVolumeMusicValue"]);
		lsTextfieldRed=(mysdl::gui::LabelStyle *)(*gui->getStyles())["lsTextfieldRed"];
		lsTextfieldBlue=(mysdl::gui::LabelStyle *)(*gui->getStyles())["lsTextfieldBlue"];

		gui->setCallback(&callback);

		labelHeaderVersion->setCaption(getFullVersion());
		updateFullscreenButtons();

		mysdl::basic_Vector2D<GLfloat> dimFull=mysdl::vpScreen.dimensions;
		mysdl::basic_Vector2D<GLfloat> dimMenu=mysdl::vpScreen.dimensions;
		dimMenu.y=logoPosition.y;

		paneMain->center(dimMenu);
		paneSettings->center(dimMenu);
		paneCredits->center(dimMenu);
		paneControls->center(dimFull);
		paneGetKey->center(dimFull);
		paneSoundSettings->center(dimMenu);
		paneEnterName->center(dimFull);
		paneInGame->center(dimFull);
		paneQuit->center(dimMenu);

		imageLoader=new mysdl::gfx::ImageLoader(imagesGame,this);
		mysdl::backgroundTasks.push_back(imageLoader);
		((MaxFighter*)(mysdl::application))->desiredTicks=0;
	}

	/*
	* See class definition.
	*/
	void Resources::initMenu()
	{
		// do nothing
	}

	/*
	* See class definition.
	*/
	void Resources::uninitMenu()
	{
		// do nothing
	}

	/*
	* See class definition.
	*/
	void Resources::initGame()
	{
		imagesGame->refresh(0);
	}

	/*
	* See class definition.
	*/
	void Resources::uninitGame()
	{
		// do nothing
	}

	/*
	* See class definition.
	*/
	void Resources::loadSettings()
	{
		mysdl::xml::Node *root=0;

		mysdl::debug::debug(dbgResources,"Loading settings from file \"%s\".",(dirManager.getSubDir(mysdl::sdUser,"",false)+FILE_SETTINGS).c_str());

		try
		{
			root=mysdl::xml::loadXML((dirManager.getSubDir(mysdl::sdUser,"",false)+FILE_SETTINGS).c_str());
		}
		catch(std::string ex)
		{
			mysdl::debug::warning(dbgResources,"Could not load settings: %s",ex.c_str());
			return;
		}

		if(root)
		{
			for(mysdl::xml::Nodes::const_iterator itr=root->children.begin(); itr!=root->children.end(); itr++)
			{
				const mysdl::xml::Node *node=*itr;

				if(node->name==XML_SETTINGS_CONTROLS)
				{
					std::string name=node->getAttribute(XML_SETTINGS_NAME,"");

					if(name==XML_SETTINGS_CONTROLS_SINGLE)
					{
						evaluatorPlayerSingle[0]->getTranslatorsFromNode(node);
					}
					else if(name==XML_SETTINGS_CONTROLS_MULTI_1)
					{
						evaluatorPlayerMulti[0]->getTranslatorsFromNode(node);
					}
					else if(name==XML_SETTINGS_CONTROLS_MULTI_2)
					{
						evaluatorPlayerMulti[1]->getTranslatorsFromNode(node);
					}
					else
					{
						mysdl::debug::warning(dbgResources,"Unknown control group \"%s\".",name.c_str());
					}
				}
				else if(node->name==XML_SETTINGS_SCREEN)
				{
					settings.fullscreen=node->getAttributeBool(XML_SETTINGS_FULLSCREEN,settings.fullscreen);
					settings.xRes=node->getAttributeLong(XML_SETTINGS_XRES,settings.xRes);
					settings.yRes=node->getAttributeLong(XML_SETTINGS_YRES,settings.yRes);
				}
				else if(node->name==XML_SETTINGS_SOUND)
				{
					settings.volumeSfx=node->getAttributeLong(XML_SETTINGS_VOLUME_SFX,settings.volumeSfx);
					settings.volumeMusic=node->getAttributeLong(XML_SETTINGS_VOLUME_MUSIC,settings.volumeMusic);
				}
			}

			delete root;
		}
	}

	/*
	* See class definition.
	*/
	void Resources::saveSettings()
	{
		mysdl::xml::Node *root=new mysdl::xml::Node();
		root->name="settings";

		mysdl::xml::Node *node;

		node=new mysdl::xml::Node();
		node->name=XML_SETTINGS_SCREEN;
		node->setAttributeBool(XML_SETTINGS_FULLSCREEN,settings.fullscreen);
		node->setAttributeLong(XML_SETTINGS_XRES,settings.xRes);
		node->setAttributeLong(XML_SETTINGS_YRES,settings.yRes);
		(*root)+=node;

		node=new mysdl::xml::Node();
		node->name=XML_SETTINGS_SOUND;
		node->setAttributeLong(XML_SETTINGS_VOLUME_SFX,settings.volumeSfx);
		node->setAttributeLong(XML_SETTINGS_VOLUME_MUSIC,settings.volumeMusic);
		(*root)+=node;

		node=new mysdl::xml::Node();
		node->name=XML_SETTINGS_CONTROLS;
		(*node)[XML_SETTINGS_NAME]=XML_SETTINGS_CONTROLS_SINGLE;
		evaluatorPlayerSingle[0]->addTranslatorsToTree(node);
		(*root)+=node;

		node=new mysdl::xml::Node();
		node->name=XML_SETTINGS_CONTROLS;
		(*node)[XML_SETTINGS_NAME]=XML_SETTINGS_CONTROLS_MULTI_1;
		evaluatorPlayerMulti[0]->addTranslatorsToTree(node);
		(*root)+=node;

		node=new mysdl::xml::Node();
		node->name=XML_SETTINGS_CONTROLS;
		(*node)[XML_SETTINGS_NAME]=XML_SETTINGS_CONTROLS_MULTI_2;
		evaluatorPlayerMulti[1]->addTranslatorsToTree(node);
		(*root)+=node;

		mysdl::debug::debug(dbgResources,"Saving settings to file \"%s\".",(dirManager.getSubDir(mysdl::sdUser,"",true)+FILE_SETTINGS).c_str());
		root->saveXML((dirManager.getSubDir(mysdl::sdUser,"",true)+FILE_SETTINGS).c_str());
		delete root;
	}

	/*
	* See class definition.
	*/
	void Resources::setVolumeSfx(int volume)
	{
		currentVolumeSfx=volume;

		sourcesGeneral->setVolume(((ALfloat)volume)/100.0);
		sourcesExplosions->setVolume(((ALfloat)volume)/100.0);
	}

	/*
	* See class definition.
	*/
	void Resources::setVolumeMusic(int volume)
	{
		currentVolumeMusic=volume;

		if(musicManager)
		{
			musicManager->setVolume(((ALfloat)volume)/100.0);
			musicManager->setCurrentVolume(((ALfloat)volume)/100.0);
		}
	}

	/*
	* See class definition.
	*/
	void Resources::updateFullscreenButtons()
	{
		cbFullscreen->setChecked(settings.fullscreen);
		cbInGameFullscreen->setChecked(settings.fullscreen);
	}

	// ==================================================================
	//  HighscoreEntry
	// ==================================================================

	/*
	* See class definition.
	*/
	HighscoreEntry::HighscoreEntry() : name(),score(0)
	{
	}

	/*
	* See class definition.
	*/
	HighscoreEntry::HighscoreEntry(std::string name,unsigned int score) : name(name),score(score)
	{
	}

	/*
	* See class definition.
	*/
	bool HighscoreEntry::operator>(const HighscoreEntry &e) const
	{
		return(score>e.score);
	}

	/*
	* See class definition.
	*/
	void HighscoreEntry::getFromNode(mysdl::xml::Node *node)
	{
		name=node->getAttribute("name","");
		score=node->getAttributeLong("score",0);
	}

	/*
	* See class definition.
	*/
	void HighscoreEntry::putInNode(mysdl::xml::Node *node)
	{
		(*node)["name"]=name;
		(*node)["score"]=mysdl::toStr<long>(score);
	}

	// ==================================================================
	//  Highscores
	// ==================================================================

	/*
	* See class definition.
	*/
	void Highscores::clear()
	{
		singleA.clear();
		singleB.clear();
		multi.clear();
	}

	/*
	* See class definition.
	*/
	void Highscores::getFromNode(const mysdl::xml::Node *root)
	{
		for(mysdl::xml::Nodes::const_iterator itr=root->children.begin(); itr!=root->children.end(); itr++)
		{
			mysdl::xml::Node *node=*itr;

			if(node->name==XML_HIGHSCORES_SINGLE_A)
			{
				singleA.getFromNode(node);
			}
			else if(node->name==XML_HIGHSCORES_SINGLE_B)
			{
				singleB.getFromNode(node);
			}
			else if(node->name==XML_HIGHSCORES_MULTI)
			{
				multi.getFromNode(node);
			}
		}
	}

	/*
	* See class definition.
	*/
	mysdl::xml::Node *Highscores::buildDomTree()
	{
		mysdl::xml::Node *root=new mysdl::xml::Node(XML_HIGHSCORES.c_str());

		mysdl::xml::Node *node;

		node=singleA.buildDomTree();
		node->name=XML_HIGHSCORES_SINGLE_A;
		(*root)+=node;

		node=singleB.buildDomTree();
		node->name=XML_HIGHSCORES_SINGLE_B;
		(*root)+=node;

		node=multi.buildDomTree();
		node->name=XML_HIGHSCORES_MULTI;
		(*root)+=node;

		return(root);
	}

	/*
	* See class definition.
	*/
	void Highscores::load(std::string filename)
	{
		mysdl::xml::Node *root=mysdl::xml::loadXML(filename.c_str());

		if(root)
		{
			if(root->name==XML_HIGHSCORES)
			{
				getFromNode(root);
			}

			delete root;
		}
	}

	/*
	* See class definition.
	*/
	void Highscores::save(std::string filename)
	{
		mysdl::xml::Node *root=buildDomTree();
		root->saveXML(filename.c_str());
		delete root;
	}

	// ==================================================================
	//  GameState
	// ==================================================================

	/*
	* See class definition.
	*/
	GameState::GameState() :
		mysdl::ApplicationState(),
		paused(false),
		originalEvaluator(0),editEvaluator(0),
		editTrigger(0),
		controlsChanged(false),currentControlsChanged(false),currentControlsButton(0),
		highscoreHighlightAngle(0)
	{
		performStageCount=3;
		drawStageCount=3;
	}

	/*
	* See class definition.
	*/
	void GameState::doDraw(int stage)
	{
		switch(stage)
		{
			case 0:
			{
				resources->contextGame.apply();
				mysdl::clearScreen();
				mysdl::resetCursor();

				resources->starfield->draw();

				break;
			}

			case 2:
			{
				static mysdl::Color color(1.0,1.0,1.0,0.5);

				resources->contextMenu.apply();
				mysdl::resetCursor();

				if(resources->gui->getVisible())
				{
					resources->gui->draw();
				}

				resources->contextGame.apply(); // TODO: don't?

				//mysdl::perform(mysdl::getCurrentTicks()+20); // +20 to allow some time for loading images
				mysdl::perform(mysdl::getCurrentTicks()+(resources->imagesLoaded ? 0 : 20)); // +20 to allow some time for loading images

				mysdl::flipScreen();

				break;
			}
		}
	}

	/*
	* See class definition.
	*/
	void GameState::updateKeyLabels()
	{
		resources->labelControlsLeft->setCaption(((*editEvaluator)["x"])->keysToString(-1.0));
		resources->labelControlsRight->setCaption(((*editEvaluator)["x"])->keysToString(+1.0));
		resources->labelControlsUp->setCaption(((*editEvaluator)["y"])->keysToString(-1.0));
		resources->labelControlsDown->setCaption(((*editEvaluator)["y"])->keysToString(+1.0));

		resources->labelControlsLeftJoy->setCaption(((*editEvaluator)["x"])->joyToString(-1,-1.0));
		resources->labelControlsRightJoy->setCaption(((*editEvaluator)["x"])->joyToString(+1,+1.0));
		resources->labelControlsUpJoy->setCaption(((*editEvaluator)["y"])->joyToString(-1,-1.0));
		resources->labelControlsDownJoy->setCaption(((*editEvaluator)["y"])->joyToString(+1,+1.0));

		resources->labelControlsFire->setCaption(((*editEvaluator)["fire"])->keysToString());
		resources->labelControlsShield->setCaption(((*editEvaluator)["shield"])->keysToString());
		resources->labelControlsTimeWarp->setCaption(((*editEvaluator)["timewarp"])->keysToString());

		resources->labelControlsFireJoy->setCaption(((*editEvaluator)["fire"])->joyToString());
		resources->labelControlsShieldJoy->setCaption(((*editEvaluator)["shield"])->joyToString());
		resources->labelControlsTimeWarpJoy->setCaption(((*editEvaluator)["timewarp"])->joyToString());
		resources->labelControlsPauseJoy->setCaption(((*editEvaluator)["pause"])->joyToString());
	}

	/*
	* See class definition.
	*/
	void GameState::showWindowControls(const std::string &title,mysdl::InputEvaluator *evaluator)
	{
		originalEvaluator=evaluator;
		editEvaluator=evaluator->clone();
		currentControlsChanged=false;

		resources->labelControlsHeader->setCaption(title);
		updateKeyLabels();

		resources->paneControls->setVisible(true);
		resources->paneInGame->setVisible(false);
		resources->paneSettings->setVisible(false);
		resources->btnControlsLeft->setFocused(true);
	}

	/*
	* See class definition.
	*/
	void GameState::closeWindowControls(bool confirm)
	{
		if(editEvaluator)
		{
			if(confirm)
			{
				originalEvaluator->copyFrom(editEvaluator);
			}

			controlsChanged|=currentControlsChanged;

			delete editEvaluator;
		}

		resources->paneControls->setVisible(false);
	}

	/*
	* See class definition.
	*/
	void GameState::showWindowSoundSettings()
	{
		resources->sVolumeSounds->value=resources->settings.volumeSfx;
		resources->sVolumeMusic->value=resources->settings.volumeMusic;
		resources->lVolumeSounds->setCaption(mysdl::toStr(resources->settings.volumeSfx));
		resources->lVolumeMusic->setCaption(mysdl::toStr(resources->settings.volumeMusic));

		resources->paneSoundSettings->setVisible(true);
		resources->paneInGame->setVisible(false);
		resources->paneSettings->setVisible(false);
		resources->btnSoundSettingsOk->setFocused(true);
	}
	/*
	* See class definition.
	*/
	void GameState::closeWindowSoundSettings(bool confirm)
	{
		resources->paneSoundSettings->setVisible(false);

		if(confirm)
		{
			resources->settings.volumeSfx=resources->currentVolumeSfx;
			resources->settings.volumeMusic=resources->currentVolumeMusic;

			if((resources->musicManager)&&(resources->settings.volumeMusic==0))
			{
				resources->musicManager->stop();
				delete resources->musicManager;
				resources->musicManager=0;
			}
		}
		else
		{
			resources->setVolumeSfx(resources->settings.volumeSfx);
			resources->setVolumeMusic(resources->settings.volumeMusic);
		}
	}

	/*
	* See class definition.
	*/
	bool GameState::closeWindows(bool confirm)
	{
		if(resources->paneSoundSettings->getVisible())
		{
			closeWindowSoundSettings(confirm);
			return true;
		}
		else if(resources->paneControls->getVisible())
		{
			closeWindowControls(confirm);
			return true;
		}

		return false;
	}

	/*
	* See class definition.
	*/
	void GameState::handleEvent(const mysdl::Trigger *e)
	{
		switch(e->id)
		{
			case cgToggleFullscreen:
			{
				resources->settings.fullscreen=!resources->settings.fullscreen;

				#ifdef win32
				if(resources->imagesLoaded)
				{
					resources->setResolution(resources->settings.xRes,resources->settings.yRes,32,resources->settings.fullscreen);
				}
				#else
				resources->setResolution(resources->settings.xRes,resources->settings.yRes,32,resources->settings.fullscreen);
				#endif

				resources->updateFullscreenButtons();

				break;
			}

			case cgScreenshot:
			{
				mysdl::saveScreenshot(FILE_SCREENSHOT);
				break;
			}

			case cmSoundSettings:
			{
				showWindowSoundSettings();
				break;
			}

			case cmVolumeSounds:
			{
				int volume=resources->sVolumeSounds->value;

				if(volume!=resources->currentVolumeSfx)
				{
					resources->setVolumeSfx(volume);
					resources->sourcesGeneral->play(resources->sampleMenuMove);
					resources->lVolumeSounds->setCaption(mysdl::toStr(volume));
				}
			}

			case cmVolumeMusic:
			{
				int volume=resources->sVolumeMusic->value;

				if(volume!=resources->currentVolumeMusic)
				{
					resources->sourcesGeneral->play(resources->sampleMenuMove);

					if((volume>0)&&((!resources->musicManager)||(!(resources->musicManager->isPlaying()))))
					{
						if(!resources->musicManager)
						{
							resources->musicManager=new mysdl::audio::MusicManager();
						}
						
						startMusic();
					}
				
					resources->setVolumeMusic(volume);
					resources->lVolumeMusic->setCaption(mysdl::toStr(volume));
				}
			}

			case cmControlsSinglePlayer:
			case cmControlsMultiplayer:
			{
				if(e->id==cmControlsSinglePlayer)
				{
					showWindowControls("Single Player Controls",resources->evaluatorPlayerSingle[e->data]);
				}
				else if(e->id==cmControlsMultiplayer)
				{
					showWindowControls("Multiplayer Controls - Player "+mysdl::toStr(e->data+1),resources->evaluatorPlayerMulti[e->data]);
				}

				break;
			}

			case cmControlsKey:
			{
				currentControlsButton=resources->gui->getFocused();
				newKeyValue=e->data;
				editTrigger=(*editEvaluator)[e->dataString];
				
				if(editTrigger->id==cgPlayerX)
				{
					resources->labelGetKeyTriggerName->setCaption(newKeyValue<0  ? "Left" : "Right");
				}
				else if(editTrigger->id==cgPlayerY)
				{
					resources->labelGetKeyTriggerName->setCaption(newKeyValue<0  ? "Up" : "Down");
				}
				else
				{
					resources->labelGetKeyTriggerName->setCaption(editTrigger->humanName);
				}

				if(editTrigger->id==cgExit)
				{
					resources->labelGetKeyHeader->setVisible(false);
					resources->labelGetKeyHeaderAlternate->setVisible(true);
				}
				else
				{
					resources->labelGetKeyHeader->setVisible(true);
					resources->labelGetKeyHeaderAlternate->setVisible(false);
				}
				
				resources->paneGetKey->setVisible(true);
				resources->icGetKey->setEnabled(true);
				resources->icGetKey->setFocused(true);

				break;
			}

			case cmGetKey:
			{
				if(resources->icGetKey->getKey()!=SDLK_ESCAPE)
				{
					editTrigger->clearKeyTranslators(newKeyValue);
					
					if(resources->icGetKey->getKey()==SDLK_BACKSPACE)
					{
						editTrigger->clearJoyButtonTranslators(newKeyValue);
						editTrigger->clearJoyAxisTranslators((short int)newKeyValue);
					}
					else if(editTrigger->id==cgExit) // =pause game
					{
						resources->icGetKey->setEnabled(true);
						resources->icGetKey->setFocused(true);
						break;
					}
					else
					{
						new mysdl::KeyTranslator(editTrigger,resources->icGetKey->getKey(),mysdl::ctrlDefault,newKeyValue);
						currentControlsChanged=true;
					}
					
					updateKeyLabels();
				}

				resources->paneControls->setVisible(true);
				resources->paneGetKey->setVisible(false);
				currentControlsButton->setFocused(true);

				break;
			}

			case cmGetJoyButton:
			{
				editTrigger->clearJoyButtonTranslators(newKeyValue);
				editTrigger->clearJoyAxisTranslators((short int)newKeyValue);
				new mysdl::JoyButtonTranslator(editTrigger,resources->icGetKey->getJoy(),resources->icGetKey->getJoyButton(),mysdl::ctrlDefault,newKeyValue);
				currentControlsChanged=true;
				updateKeyLabels();

				resources->paneControls->setVisible(true);
				resources->paneGetKey->setVisible(false);
				currentControlsButton->setFocused(true);

				break;
			}

			case cmGetJoyAxis:
			{
				editTrigger->clearJoyButtonTranslators(-1);
				editTrigger->clearJoyButtonTranslators(+1);
				editTrigger->clearJoyAxisTranslators(-1);
				editTrigger->clearJoyAxisTranslators(+1);
				new mysdl::JoyAxisTranslator(editTrigger,resources->icGetKey->getJoy(),resources->icGetKey->getJoyAxis(),-1,4000,30000);
				new mysdl::JoyAxisTranslator(editTrigger,resources->icGetKey->getJoy(),resources->icGetKey->getJoyAxis(),+1,4000,30000);
				currentControlsChanged=true;
				updateKeyLabels();

				resources->paneControls->setVisible(true);
				resources->paneGetKey->setVisible(false);
				currentControlsButton->setFocused(true);

				break;
			}
		}
	}

	/*
	* See class definition.
	*/
	void GameState::doPerform(int stage,Uint32 ticks)
	{
		switch(stage)
		{
			case 0:
			{
				lastFps=mysdl::calcFPS((int)ticks);

				highscoreHighlightAngle+=0.001*ticks;

				while(highscoreHighlightAngle>3.141592654)
				{
					highscoreHighlightAngle-=3.141592654;
				}

				resources->sourcesExplosions->perform(ticks);

				resources->queue->perform(ticks);
				resources->gui->Component::perform(resources->queue,ticks);
			}

			case 2:
			{
				for(std::list<mysdl::sprites::Sprite *>::iterator itr=deleteQueue.begin(); itr!=deleteQueue.end(); itr++)
				{
					delete *itr;
				}

				deleteQueue.clear();

				if(!paused)
				{
					resources->starfield->perform(ticks);
				}
			}
		}
	}

	/*
	* See class definition.
	*/
	void GameState::displayHighscores(GLfloat xOffset,GLfloat yOffset,HighscoreTable *highscores,int posRed,int posBlue)
	{
		GLfloat highlight=sin(highscoreHighlightAngle);

		mysdl::Color highscoreRed(1.0,highlight,highlight,1.0);
		mysdl::Color highscoreBlue(highlight,highlight,1.0,1.0);
		mysdl::Color yellow(1.0,1.0,0.0,1.0);

		for(int i=0; i<10; i++)
		{
			if(i==posRed)
			{
				highscoreRed.apply();
			}
			else if(i==posBlue)
			{
				highscoreBlue.apply();
			}
			else
			{
				yellow.apply();
			}

			GLfloat y=yOffset+5*30-i*30;

			mysdl::resetCursor();
			mysdl::moveCursor(mysdl::Vector2D(xOffset-105,y));
			resources->fontStandard->printf(mysdl::jhRight,"%d.",(i+1));

			mysdl::resetCursor();
			mysdl::moveCursor(mysdl::Vector2D(xOffset-100,y));
			resources->fontStandard->printf(mysdl::jhLeft,"%s",highscores->entries[i].name.c_str());

			mysdl::resetCursor();
			mysdl::moveCursor(mysdl::Vector2D(xOffset+130,y));
			resources->fontStandard->printf(mysdl::jhRight,"%d",highscores->entries[i].score);
		}
	}

	// ==================================================================
	//  GUICallback
	// ==================================================================

	/*
	* See class definition.
	*/
	void GUICallback::onMove()
	{
		if(menu)
		{
			menu->stateChangeTimer=specs.menuSwitchDuration;
		}

		resources->sourcesGeneral->play(resources->sampleMenuMove);
	}

	/*
	* See class definition.
	*/
	void GUICallback::onConfirm()
	{
		if(menu)
		{
			menu->stateChangeTimer=specs.menuSwitchDuration;
		}

		resources->sourcesGeneral->play(resources->sampleMenuClick);
	}
};
