/**
 * Max Fighter
 * Copyright (C) 2006 Marian Schedenig (marian@musgit.com)
 * http://maxfighter.musgit.com/
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
#include <math.h>

#include "mysdl/debug.h"

#include "specs.h"

namespace maxfighter
{
	/** Returns the maximum of two values. */
	#define max(x,y) ((x)>(y) ? (x) : (y))

	/** Global (constant) instance containing game specifications. */
	Specs specs;

	/*
	 * See class definition.
	 */
	Specs::Specs() :
		maxTicksPerFrame(100),
		desiredTicks(1000/100),
		desiredTicksEpsilon(0),
		playfieldBorderX(20),
		startY(-50),
		standardY(50),
		maxYModeA(500),
		maxYModeB(500),
		minPlayerDistanceX(90),
		minPlayerDistanceY(75),
		playerSpeedFactorY(0.8),
		baseSpeed(0.2),
		speedStart(0.35),
		speedIncrement(0.04),
		minLevelForShield(1),
		minLevelForWeapons(2),
		splitSpeedX(0.2),
		starCount(1500),
		starSpeedFactor(0.8),
		speedupCountMax(5),
		fireRateCountMax(5),
		laserLevelMax(4),
		containerSpeedFactor(1.0),
		extraSpeedFactor(0.7),
		extraFadeFactor(0.003),
		playerTiltAnimFrames(5),
		shieldEnergyPacket(2500),
		shieldEnergyMax(15000),
		timeWarpLevelMax(2),
		shieldFadeInFactor(0.01),
		shieldFadeOutFactor(0.001),
		shieldCollapseFactor(0.02),
		timeWarpFactor(0.4),
		timeWarpStartSpeed(0.001),
		timeWarpStopSpeed(0.001),
		timeWarpDuration(7000),//10000),
		containerScoreStep(5000),
		starfieldMenuSpeed(0.03),
		starfieldMenuSpeedStep(0.0002),
		extraVelocityFactor(0.3),
		extraIntervalScore(5000),
		extraIntervalExtra(10000),
		scoreIncrease(1),
		speedIncreaseB(0.001),
		asteroidIntervalDecreaseB(1),
		asteroidIntervalMinB(80),
		chanceLargeAsteroidB(0.05),
		extraReviveTicks(200),
		extraContainerLimit(0),
		asteroidTargetY(200),
		hudSwitchTime(250),
		hudSwitchCount(10),
		shieldConsumption(2.0),
		restartDelay(2000),//100),
		scoreCrystal(500),
		scoreFoe(1500),
		scoreAsteroidSmall(50),
		scoreAsteroidLarge(100),
		scoreBonus(5000),
		foeFirstLevel(3),
		foeSpeed(0.3),
		foeLaserSpeed(0.4),
		foeLaserSpeedVertical(0.4),
		foeTop(450),
		foeFloor(60),
		foeLaserFadeOutSpeed(0.005),
		startSpeedModeB(0.4),//5),
		startAsteroidIntervalModeB(400),
		menuSwitchDuration(30*1000),
		menuHighscoreDuration(20*1000),
		menuHelpStepDuration(60*1000),
		hudPosPlayer1(mysdl::Vector2D(740,540)),
		hudPosPlayer2(mysdl::Vector2D(740,540)),
		colorHudBack(0.25,0.25,0.25,1.0),
		colorHudFront(0.75,0.75,0.75,1.0)
	{
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getAsteroidInterval(Uint32 level)
	{
		static Uint32 lastLevel=0;
		static Uint32 lastInterval=1;

		if(level==lastLevel)
		{
			return lastInterval;
		}

		GLfloat interval=550;

		for(Uint32 i=1; i<=level; i++)
		{
			interval-=230.0/(GLfloat)(i*i);
		}

		lastLevel=level;
		lastInterval=(Uint32)interval;

		return lastInterval;
	}

	/*
	* See class definition.
	*/
	mysdl::Vector2D Specs::getAsteroidVelocity(GLfloat speed)
	{
		mysdl::Vector2D v=mysdl::Vector2D(mysdl::random<GLfloat>(-0.50*speed,0.50*speed),-mysdl::random<GLfloat>(0.03,speed));
		return(v);
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getSpeedForLevel(Uint32 level)
	{
		return 0.3+((GLfloat)level)*0.027;
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getKillsForLevel(Uint32 level)
	{
		Uint32 i=level-1;
		return 40+i*i;
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::speedupInterval(GLfloat speed)
	{
		speed*=4;
		return(25+(Uint32)(75*(speed*speed*speed)));
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getLaserDelay(short int level)
	{
		static short int lastLevel=-1;
		static GLfloat lastDelay=1;

		if(level==lastLevel)
		{
			return lastDelay;
		}

		lastDelay=500;

		for(short int i=1; i<=level; i++)
		{
			lastDelay-=120.0/(GLfloat)(i);
		}

		lastLevel=level;

		return lastDelay;
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getLaserSpeed(short int level)
	{
		static short int lastLevel=-1;
		static GLfloat lastSpeed=1;

		if(level==lastLevel)
		{
			return lastSpeed;
		}

		lastSpeed=0.5;

		for(short int i=1; i<=level; i++)
		{
			lastSpeed+=0.13/(GLfloat)(i);
		}
		lastLevel=level;

		return lastSpeed;
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getChanceLargeAsteroid(short int level)
	{
		if(level<=3)
		{
			return 0.0;
		}
		else
		{
			return (level-3)*0.07;
		}
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getChanceAsteroidEnergy(short int level)
	{
		static short int lastLevel=-1;
		static GLfloat lastChance=0;

		if(level==lastLevel)
		{
			return lastChance;
		}

		lastChance=0.0;

		if(level>=5)
		{
			for(short int i=1; i<=level-5; i++)
			{
				lastChance+=0.3/(GLfloat)(i);
			}
		}

		lastLevel=level;

		return lastChance;
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getMaxAsteroidEnergy(short int level)
	{
		if(level<5)
		{
			return 0;
		}
		else
		{
			return mysdl::min((level-3)*1,10);
		}
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getFoeFireDelay(short int level)
	{
		static short int lastLevel=-1;
		static Uint32 lastDelay=0;

		if(level==lastLevel)
		{
			return lastDelay;
		}

		GLfloat delay=3000.0;

		for(short int i=1; i<=level; i++)
		{
			delay-=400.0/(GLfloat)(i);
		}

		lastDelay=(Uint32)delay;
		lastLevel=level;

		return lastDelay;
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getFoeVolleyCount(short int level)
	{
		if(level<5)
		{
			return 0;
		}
		else
		{
			return mysdl::random_int((level-5)/2);
		}
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getFoeVolleyDelay(short int level)
	{
		static short int lastLevel=-1;
		static Uint32 lastDelay=0;

		if(level==lastLevel)
		{
			return lastDelay;
		}

		GLfloat delay=1500.0;

		for(short int i=1; i<=level; i++)
		{
			delay-=400.0/(GLfloat)(i);
		}

		lastDelay=(Uint32)delay;
		lastLevel=level;

		return lastDelay;
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getFoeHull(short int level)
	{
		if(level<4)
		{
			return 50;//30;
		}
		else
		{
			return 50/*30*/+mysdl::random_int((level-4)*10);
		}
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getFoeAcceleration(short int level)
	{
		return mysdl::random<GLfloat>(0.002,0.004);
	}

	/*
	* See class definition.
	*/
	GLfloat Specs::getFoeDistanceSpeedFactor(short int level)
	{
		return 5.0+mysdl::random<float>(2*level);
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getFoeLaserTtl(short int level)
	{
		return mysdl::min<Uint32>(1500,400+mysdl::random_int(70*level));
	}

	/*
	* See class definition.
	*/
	int Specs::getFoeKamikazeChance(short int level)
	{
// 		return mysdl::max<int>(200,1500-12*level);
		return mysdl::max<int>(200,1500-20*level);
	}

	/*
	* See class definition.
	*/
	int Specs::getFoeFloorChance(short int level)
	{
// 		return mysdl::max<int>(200,1500-15*level);
		return mysdl::max<int>(200,1500-20*level);
	}

	/*
	* See class definition.
	*/
	Uint32 Specs::getFoeFloorDuration(short int level)
	{
		return mysdl::random_int((int)level*1000);
	}
};
